@@ -4,14 +4,14 @@
 ## Author:      Graciliano M. P.
 ## Modified by: Geoffrey Leach
 ## Created:     11/01/2004
-## Updated:	    2010-06-13
-## Copyright:   (c) 2004 Graciliano M. P. (c) 2007 - 2013 Geoffrey Leach
+## Updated:	    2008-05-31
+## Copyright:   (c) 2004 Graciliano M. P. (c) 2007, 2008 Geoffrey Leach
 ## Licence:     This program is free software; you can redistribute it and/or
 ##              modify it under the same terms as Perl itself
 #############################################################################
 
 package Pod::HtmlEasy::Parser;
-use 5.006003;
+use 5.006002;
 
 use base qw{ Pod::Parser };
 use Pod::Parser;
@@ -24,54 +24,41 @@ use English qw{ -no_match_vars };
 use Regexp::Common qw{ whitespace number URI };
 use Regexp::Common::URI::RFC2396 qw { $escaped };
 use Pod::Escapes qw{ e2char };
-
-
-# Provided for RT 82400. Use native switch if available.
-BEGIN {
-        if ($PERL_VERSION >= 5.012) {
-                require feature;
-                "feature"->import(qw(switch));
-        } else {
-                require Switch;
-                "Switch"->import(qw(Perl6));
-        }
-}
+use Switch qw{ Perl6 };
 
 use strict;
 use warnings;
 
+use version; our $VERSION = qv("1.0.1");
+
 ########
 # VARS #
 ########
 
 Readonly::Scalar my $NUL => NUL;
 
-# RT 58274 [\w-]+ => [\w\.-]
-# Commented patterns temp test
 Readonly::Scalar my $MAIL_RE => qr{
          (         # grab all of this
-         [\w\.-]+  # some word chars with '-' and '.'included  foo
+         [\w-]+    # some word chars with '-' included   foo
          \0?       # possible NUL escape
-         \@        # literal '@'                               @
-         [\w\.-]+  # another word                              bar
+         \@        # literal '@'                         @
+         [\w\\-]+  # another word                        bar
          (?:       # non-grabbing pattern
-#         \.       # literal '.'                               .
-          [\w\.-]+ # that word stuff                           stuff
-#         \.       # another literal '.'                       .
-          [\w\.-]+ # another word                              and
+          \.       # literal '.'                        .
+          [\w\-\.]+# that word stuff                    stuff
+          \.       # another literal '.'                .
+          [\w\-]+  # another word                       and
           |        # or
-#         \.       # literal '.'                               .   
-          [\w\.-]+ # word                                      nonsense
+          \.       # literal '.'                        .   
+          [\w\-]+  # word                               nonsense
           |        # or empty?
-         )         # end of non-grab
-         )         # end of grab
+         )        # end of non-grab
+         )        # end of grab
         }smx;    # [6062]
 
-
-
 # Treatment of embedded HTML-significant characters and embedded URIs.
 
-# There are some characters (%HTML_ENTITIES below) which may in some
+# There are some characters (%html_entities below) which may in some
 # circumstances be interpreted by a browser, and you probably don't want that
 # Consequently, they are replaced by names defined by the W3C UNICODE spec,
 # http://www.w3.org/TR/MathML2/bycodes.html, bracketed by '&' and ';'
@@ -99,15 +86,15 @@ Readonly::Scalar my $MAIL_RE => qr{
 # the generated URI, but after the first character. These NULs are removed
 # by _remove _nul_escapes()
 
-Readonly::Hash my %HTML_ENTITIES => (
+Readonly::Hash my %html_entities => (
     q{&} => q{amp},
     q{>} => q{gt},
     q{<} => q{lt},
     q{"} => q{quot},
 );
 
-my $HTML_ENTITIES_RE = join q{|}, keys %HTML_ENTITIES;
-$HTML_ENTITIES_RE = qr{$HTML_ENTITIES_RE}msx;
+my $HTML_ENTITIES_RE = join q{|}, keys %html_entities;
+$HTML_ENTITIES_RE = qr{$HTML_ENTITIES_RE}mx;
 
 #################
 # _NUL_ESCAPE   #
@@ -138,15 +125,13 @@ sub _remove_nul_escapes {
 ####################
 
 sub _encode_entities {
-    my $txt_ref = shift;
+    my ( $parser, $txt_ref ) = @_;
 
-    if ( !( defined $txt_ref && length ${$txt_ref} ) ) { return; }
-
-    foreach my $chr ( keys %HTML_ENTITIES ) {
+    foreach my $chr ( keys %html_entities ) {
 
         # $chr gets a lookbehind to avoid converting flagged from E<...>
         my $re = qq{(?<!$NUL)$chr};
-        ${$txt_ref} =~ s{$re}{$NUL&$HTML_ENTITIES{$chr};}gsmx;
+        ${$txt_ref} =~ s{$re}{$NUL&$html_entities{$chr};}gsmx;
     }
 
     return;
@@ -156,14 +141,14 @@ sub _encode_entities {
 # _ADD_URI_HREF #
 #################
 
-# process embedded URIs that are not noted in L<...> bracketing
+# process embedded URIs that are not noted in l<...> bracketing
 # Note that the HTML-significant characters are escaped;
 # The escapes are removed by _encode_entities
 # Note that there's no presumption that there's a URI in the
 # text, so not matching is _not_ and error.
 
 sub _add_uri_href {
-    my ($txt_ref) = @_;
+    my ( $parser, $txt_ref ) = @_;
 
     if ( ${$txt_ref} =~ m{https?:}smx ) {
 
@@ -172,12 +157,12 @@ sub _add_uri_href {
 # per the RFC. However, the Spamassassin folks use it in the host.
 # $escaped is defined by Regexp::Common::URI::RFC2396, and matches %xx
 # This is done first because if needed, the host part won't be parsed correctly
-        while ( ${$txt_ref} =~ m{($escaped)}msx ) {
+        while ( ${$txt_ref} =~ m{($escaped)}mx ) {
             my $esc = $1;
             my $new = $1;
-            $new =~ s{%}{0x}msx;
+            $new =~ s{%}{0x}mx;
             $new = e2char($new);
-            ${$txt_ref} =~ s{$esc}{$new}gmsx;
+            ${$txt_ref} =~ s{$esc}{$new}gmx;
         }
 
    # target='_blank' causes load to a new window or tab
@@ -237,68 +222,63 @@ sub command {
     $expansion =~ s{$RE{ws}{crop}}{}gsmx;    # delete surrounding whitespace
 
     # Encoding puts in a NUL; we're finished with the text, so remove them
-    _encode_entities( \$expansion );
+    _encode_entities( $parser, \$expansion );
     _remove_nul_escapes( \$expansion );
 
     my $html;
-    no warnings; # 'experimental'
     given ($command) {
-        when (q{head1}) {
+        when q{head1} {
             _add_index( $parser, $expansion, $LEVEL1 );
             $html = $parser->{POD_HTMLEASY}
                 ->{ON_HEAD1}( $parser->{POD_HTMLEASY}, $expansion );
         }
-        when (q{head2}) {
+        when q{head2} {
             _add_index( $parser, $expansion, $LEVEL2 );
             $html = $parser->{POD_HTMLEASY}
                 ->{ON_HEAD2}( $parser->{POD_HTMLEASY}, $expansion );
         }
-        when (q{head3}) {
+        when q{head3} {
             _add_index( $parser, $expansion, $LEVEL3 );
             $html = $parser->{POD_HTMLEASY}
                 ->{ON_HEAD3}( $parser->{POD_HTMLEASY}, $expansion );
         }
-        when (q{head4}) {
+        when q{head4} {
             _add_index( $parser, $expansion, $LEVEL4 );
             $html = $parser->{POD_HTMLEASY}
                 ->{ON_HEAD4}( $parser->{POD_HTMLEASY}, $expansion );
         }
-        when (q{begin}) {
+        when q{begin} {
             $html = $parser->{POD_HTMLEASY}
                 ->{ON_BEGIN}( $parser->{POD_HTMLEASY}, $expansion );
         }
-        when (q{end}) {
+        when q{end} {
             $html = $parser->{POD_HTMLEASY}
                 ->{ON_END}( $parser->{POD_HTMLEASY}, $expansion );
         }
-        when (q{over}) {
+        when q{over} {
             $html = $parser->{POD_HTMLEASY}
                 ->{ON_OVER}( $parser->{POD_HTMLEASY}, $expansion );
         }
-        when (q{item}) {
+        when q{item} {
 
             # Items that begin with '* ' are ugly. Is it there for pod2man?
             # Which is not the same as _only_ '*'
-            $expansion =~ s{\A\*\s+}{}msx;
+            $expansion =~ s{\A\*\s+}{}mx;
 
             if ( $parser->{INDEX_ITEM} ) {
                 _add_index( $parser, $expansion, $LEVELL );
             }
 
             # This is for the folks who use =item to list URLs
-            if ( $expansion !~ m{<a\shref=}smx ) {
-
-                # The URI's not already encoded (L<...> is already processed)
-                _add_uri_href( \$expansion );
-            }
+            _add_uri_href( $parser, \$expansion );
             $html = $parser->{POD_HTMLEASY}
                 ->{ON_ITEM}( $parser->{POD_HTMLEASY}, $expansion );
         }
-        when (q{back}) {
+        when q{back} {
             $html = $parser->{POD_HTMLEASY}
                 ->{ON_BACK}( $parser->{POD_HTMLEASY}, $expansion );
         }
-        when (q{for}) {
+        when q{for} {
             $html = $parser->{POD_HTMLEASY}
                 ->{ON_FOR}( $parser->{POD_HTMLEASY}, $expansion );
         }
@@ -309,13 +289,12 @@ sub command {
                     ->{qq{ON_\U$command\E}}( $parser->{POD_HTMLEASY},
                     $expansion );
             }
-            elsif ( $command !~ /^(?:pod|cut)$/imsx ) {
+            elsif ( $command !~ /^(?:pod|cut)$/imx ) {
                 $html = qq{<pre>=$command $expansion</pre>};
             }
             else { $html = EMPTY; }
         }
     };
-    use warnings;
 
     if ( $html ne EMPTY ) {
         push @{ $parser->{POD_HTMLEASY}->{HTML} }, $html;
@@ -345,25 +324,18 @@ sub _verbatim {
     my $expansion = $parser->{POD_HTMLEASY}->{VERBATIM_BUFFER};
     $parser->{POD_HTMLEASY}->{VERBATIM_BUFFER} = EMPTY;
 
-    _encode_entities( \$expansion );
-
-    # If we had "=item *", we should now be looking at the text that will
-    # appear as the item. The "*" was passed over initially, so we need
-    # the text to index. Save the flag as ON_VERBATIM deletes IN_ITEM
-
-    my $add_index = $parser->{INDEX_ITEM} && $parser->{POD_HTMLEASY}{IN_ITEM};
+    _encode_entities( $parser, \$expansion );
 
     my $html = $parser->{POD_HTMLEASY}
         ->{ON_VERBATIM}( $parser->{POD_HTMLEASY}, $expansion );
 
     # Now look for any embedded URIs
-    _add_uri_href( \$html );
+    _add_uri_href( $parser, \$html );
 
     # And remove any NUL escapes
     _remove_nul_escapes( \$html );
 
     if ( $html ne EMPTY ) {
-        if ($add_index) { _add_index( $parser, $expansion, $LEVELL ); }
         push @{ $parser->{POD_HTMLEASY}->{HTML} }, $html;
     }    # [6062]
 
@@ -389,25 +361,18 @@ sub textblock {
     $expansion =~ s{\s+$}{}gsmx;
 
     # Encode HTML-specific characters before adding any HTML (eg <p>)
-    _encode_entities( \$expansion );
-
-    # If we had "=item *", we should now be looking at the text that will
-    # appear as the item. The "*" was passed over initially, so we need
-    # the text to index. Save the flag as ON_TEXTBLOCK deletes IN_ITEM
-
-    my $add_index = $parser->{INDEX_ITEM} && $parser->{POD_HTMLEASY}{IN_ITEM};
+    _encode_entities( $parser, \$expansion );
 
     my $html = $parser->{POD_HTMLEASY}
         ->{ON_TEXTBLOCK}( $parser->{POD_HTMLEASY}, $expansion );
 
     # Now look for any embedded URIs
-    _add_uri_href( \$html );
+    _add_uri_href( $parser, \$html );
 
     # And remove any NUL escapes
     _remove_nul_escapes( \$html );
 
     if ( $html ne EMPTY ) {
-        if ($add_index) { _add_index( $parser, $expansion, $LEVELL ); }
         push @{ $parser->{POD_HTMLEASY}->{HTML} }, $html;
     }
 
@@ -424,54 +389,44 @@ sub interior_sequence {
 
     my $ret;
 
-    # Encode HTML-specific characters before adding any HTML (eg <p>)
-    if ( $seq_command ne q{L} ) {
-        _encode_entities( \$seq_argument );
-    }
-
-    no warnings; # 'experimental'
     given ($seq_command) {
-        when (q{B}) {
+        when q{B} {
             $ret = $parser->{POD_HTMLEASY}
                 ->{ON_B}( $parser->{POD_HTMLEASY}, $seq_argument );
         }
-        when (q{C}) {
+        when q{C} {
             $ret = $parser->{POD_HTMLEASY}
                 ->{ON_C}( $parser->{POD_HTMLEASY}, $seq_argument );
         }
-        when (q{E}) {
+        when q{E} {
             $ret = $parser->{POD_HTMLEASY}
                 ->{ON_E}( $parser->{POD_HTMLEASY}, $seq_argument );
         }
-        when (q{F}) {
+        when q{F} {
             $ret = $parser->{POD_HTMLEASY}
                 ->{ON_F}( $parser->{POD_HTMLEASY}, $seq_argument );
         }
-        when (q{I}) {
+        when q{I} {
             $ret = $parser->{POD_HTMLEASY}
                 ->{ON_I}( $parser->{POD_HTMLEASY}, $seq_argument );
         }
-        when (q{L}) {
+        when q{L} {
 
             # L<> causes problems, but not with parselink.
             if ( $seq_argument eq EMPTY ) {
                 _errors( $parser, q{Empty L<>} );
                 return EMPTY;
             }
-            my @parsed = Pod::ParseLink::parselink($seq_argument);
-            foreach (@parsed) {
-                if ( defined $_ ) { _encode_entities( \$_ ); }
-            }
-
-            # Encoding handled in ON_L()
-            $ret = $parser->{POD_HTMLEASY}
-                ->{ON_L}( $parser->{POD_HTMLEASY}, @parsed );
+            $ret = $parser->{POD_HTMLEASY}->{ON_L}(
+                $parser->{POD_HTMLEASY},
+                Pod::ParseLink::parselink($seq_argument)
+            );
         }
-        when (q{S}) {
+        when q{S} {
             $ret = $parser->{POD_HTMLEASY}
                 ->{ON_S}( $parser->{POD_HTMLEASY}, $seq_argument );
         }
-        when (q{Z}) {
+        when q{Z} {
             $ret = $parser->{POD_HTMLEASY}
                 ->{ON_Z}( $parser->{POD_HTMLEASY}, $seq_argument );
         }
@@ -488,7 +443,6 @@ sub interior_sequence {
             }
         }
     }
-    use warnings;
 
     # Escape HTML-significant characters
     _nul_escape( \$ret );
@@ -559,20 +513,6 @@ sub _add_index {
     # Don't index star items
     if ( $txt eq q{*} ) { return; }
 
-    if ( exists $parser->{INDEX_ITEM} ) {
-        my $max_len = $parser->{INDEX_LENGTH};
-        if ( length $txt > $max_len ) {
-            while ( substr( $txt, $max_len, 1 ) ne q{ } ) {
-                $max_len++;
-                last if $max_len >= length $txt;
-            }
-            if ( $max_len < length $txt ) {
-                $txt = substr( $txt, 0, $max_len ) . "...";
-            }
-        }
-    }
-
-    _remove_nul_escapes( \$txt );
     push @{ $parser->{POD_HTMLEASY}->{INDEX} }, [ $level, $txt ];
 
     return;
